class ArrayServer inherit DataServer

	protect lEoF			as logic
	protect lBoF			as logic
    protect nRecNo			as long
	protect nRecCount		as long
	protect aData			as array	
	protect nNotifyLevel    as short
	
method AnyID2Pos( uFieldId ) class ArrayServer

	local nPos := 0 as word
	
	do case
		case IsNumeric( uFieldId )
			nPos := uFieldId
		case IsString( uFieldId )
			nPos := ascan( self:aDataFields , {|e|e:Name == uFieldId } )
		case IsSymbol( uFieldId )		
			nPos := ascan( self:aDataFields , {|e|e:NameSym == uFieldId } )		
	endcase
	return nPos
	
access BOF() class ArrayServer
	return self:lBOF

access ConcurrencyControl class ArrayServer
	return self:nCCMode
	
assign ConcurrencyControl( nMode ) class ArrayServer
	return self:nCCMode := nMode

method DataField( uFieldId ) class ArrayServer
	local nPos as word
	nPos := self:AnyID2Pos( uFieldId )
	if nPos > 0
		return self:aDataFields[ uFieldId ]
	endif
	return nil

access DBStruct class ArrayServer
	return self:aDataFields
	
method Delete() class ArrayServer
	return true

access Deleted() class ArrayServer
	return false

access EOF() class ArrayServer
	return self:lEOF
	
access FCount class ArrayServer
	return self:wFieldCount

method FieldGet( uFieldId ) class ArrayServer

	local nPos as word
	local uResult as usual
	
	nPos := self:AnyID2Pos( uFieldId )
	
	if nPos > 0
		uResult := self:aData[ self:nRecno , nPos ]
	else
		uResult := ""
	endif
	return uResult
	
method FieldHyperLabel( uFieldId ) class ArrayServer
	local nPos as word
	nPos := self:AnyID2Pos( uFieldId )
	if nPos > 0
		return self:aDataFields[ uFieldId ]:HyperLabel
	endif
	return nil

method FieldName( nField ) class ArrayServer
	return self:aDataFields[ nField ]:Name

method FieldPut( uFieldId , uValue ) class ArrayServer

	local nPos as word
	local uResult as usual
	
	nPos := self:AnyID2Pos( uFieldId )
	
	if nPos > 0
		self:aData[ self:nRecno , nPos ] := uValue
		uResult := uValue
		self:Notify( NOTIFYFIELDCHANGE , self:aDataFields[nPos]:NameSym )		
	else
		uResult := nil
	endif
	return uResult
 
method FieldSpec( uFieldId ) class ArrayServer
	local nPos as word
	nPos := self:AnyID2Pos( uFieldId )
	if nPos > 0
		return self:aDataFields[ uFieldId ]:FieldSpec
	endif
	return nil
	
method FieldStatus( uFieldId ) class ArrayServer
	return nil
 
method FieldSym( nField ) class ArrayServer
	return self:aDataFields[ nField ]:NameSym
 
method GoBottom() class ArrayServer
	self:Notify( NOTIFYINTENTTOMOVE )
	self:GoTo( self:nRecCount )
	self:Notify( NOTIFYGOBOTTOM ) 
	return true

method GoTo( nPos ) class ArrayServer
	self:Notify( NOTIFYINTENTTOMOVE )
	self:nRecNo := nPos
	self:Notify( NOTIFYRECORDCHANGE ) 	
	return true
	
method GoTop() class ArrayServer
	self:Notify( NOTIFYINTENTTOMOVE )
	self:GoTo( 1 )
	self:Notify( NOTIFYGOTOP ) 
	return true
	
access Hyperlabel class ArrayServer
	return self:oHyperlabel
	
assign Hyperlabel( oLabel ) class ArrayServer
	return self:oHyperlabel := oLabel
	
method Init( aArray , oLabel , aDescription ) class ArrayServer

	local i as word
	local c as string
	local oHL as Hyperlabel
	local oFS as FieldSpec
	local uType as usual
		
	default( @oLabel , HyperLabel{ #ArrayServer , "ArrayServer" } )

	self:lBOF := false
	self:lEOF := false	
	self:aData       := aArray
	self:nRecCount   := ALen( self:aData )
	self:oHyperLabel := oLabel
	self:oHLStatus   := nil
	self:nCCMode     := CCOPTIMISTIC
	self:aDataFields := {}
    
	if len( self:aData ) > 0 
		if IsArray( self:aData[ 1 ] )
			self:wFieldCount := ALen( self:aData[ 1 ] )
		else
			self:wFieldCount := 1
		endif
	else
		if IsNil( aDescription )
			self:wFieldCount := 0
		else
			self:wFieldCount := ALen( aDescription )
		endif	
	endif
	
	for i := 1 to self:wFieldCount
		if aDescription == nil
			c := alltrim(str(i)) 
		else
			c := aDescription[ i ]		
		endif
		oHL := HyperLabel{ AsSymbol(c) , c }
		if IsArray( self:aData[ 1 ] )
			uType := UsualType( self:aData[ 1 , i ] ) 
		else
			uType := UsualType( self:aData[ 1 ] ) 
		endif
		oFS := FieldSpec{ oHL , uType , 10 , 0 }
		aadd( self:aDataFields , DataField{ c , oFS } )
	next
	
	self:GoTop()
	
access Name class ArrayServer
	return self:Hyperlabel:Name
	
access NameSym class ArrayServer
	return self:Hyperlabel:NameSym
	
method Notify( kNotification , uDescription ) class ArrayServer
	if self:nNotifyLevel == 0
		super:Notify( kNotification , uDescription )
	endif
	
access RecCount class ArrayServer
	return self:nRecCount

access RecNo class ArrayServer
	return self:nRecNo

assign RecNo( nRecord ) class ArrayServer
	self:Goto( nRecord )
	return self:nRecNo

method RegisterClient( oClient ) class ArrayServer
	if ascan( self:aClients , oClient ) == 0
		aadd( self:aClients , oClient )
	endif
	return true
	
method ResetNotification() class ArrayServer
	self:nNotifyLevel := max( 0 , self:nNotifyLevel - 1 )
	
method Skip( nSkip ) class ArrayServer

	self:lBOF := false
	self:lEOF := false
	
	self:Notify( NOTIFYINTENTTOMOVE )
 
	if nSkip >= 0
		if self:nRecno + nSkip <= self:nRecCount
			self:GoTo( self:nRecno + nSkip )
		else
			self:lBOF := false
			self:lEOF := true
		endif
	else
		if self:nRecno + nSkip >= 1
			self:GoTo( self:nRecno + nSkip )
		else
			self:lBOF := true
			self:lEOF := false
		endif
	endif	
	self:Notify( NOTIFYRECORDCHANGE )
	return !(self:lEOF .or. self:lBOF )
	
access Status class ArrayServer
	return self:oHLStatus
	
method SuspendNotification() class ArrayServer
	self:nNotifyLevel := self:nNotifyLevel + 1
	